# coding: utf-8
"""
Command which builds a file named configServices.js.
The js file is used to create the filter screen.
It is mainly built dynamically from the db, but some fields had to be hardcoded, because inherited by
 a previous logic.
"""
import json
import logging
import os

from django.core.management.base import BaseCommand
from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from optparse import make_option

from publieditor.content.models import Brand, Car


logger = logging.getLogger('import_news')


class Command(BaseCommand):
    """Builds configService.js from db"""
    help = 'Builds configService.js from db'
    option_list = BaseCommand.option_list + (
        make_option(
            '--debug', dest='debug',
            action='store_true',
            default=False,
            help=_('to log DEBUG too')
        ),
    )

    def create_brand_dict(self):
        """
        Creates a dict using a list of dict returned by Brand.get_brand_choices()
        The name change of the brand label "Mercedes-Benz" to "Mercedes" has been hardcode.
        "Mercedes-Benz" it's too long, it causes rendering mismatches.
        """
        brands = Brand.get_brand_choices()
        for brand in brands:
            if brand['label'] == u'Mercedes-Benz':
                brand['label'] = u'Mercedes'
        brand = dict(
            filterType='multiselect',
            paramKey='brand',
            label='Brand',
            choices=brands
        )
        return brand

    def create_price_dict(self):
        price_range = Car.get_price_range()
        price = dict(
            filterType='range',
            paramKey='price',
            label='Prezzo',
            initialRange=price_range,
            range=price_range,
            unityOfMeasure='€',
        )
        return price

    def create_shape_dict(self):
        """
        Creates a dict using a list of dict returned by Car.get_shape_choices()
        The name change of the brand label "Station Wagon" to "St. Wagon" has been hardcode.
        "Station Wagon" it's too long, it causes rendering mismatches.
        """
        shapes = Car.get_shape_choices()
        for shape in shapes:
            if shape['label'] == u'Station Wagon':
                shape['label'] = u'St. Wagon'
        shape = dict(
            filterType='multiselect',
            paramKey='shape',
            label='Design',
            choices=shapes,
        )
        return shape

    def create_target_dict(self):
        target=dict(
            filterType='multiselect',
            paramKey='target',
            label='Destinazione d\'uso',
            choices=Car.get_target_choices(),
        )
        return target

    def create_seat_dict(self):
        seat = dict(
            filterType='multiselect',
            paramKey='seat',
            label='Posti',
            choices=Car.get_seat_choices(),
        )
        return seat

    def create_engine_dict(self):
        engine = dict(
            filterType='multiselect',
            paramKey='engine',
            label='Tipo di motore',
            choices=Car.get_engine_choices(),
        )
        return engine

    def create_layout_dict(self):
        layout = dict(
            filterType='multiselect',
            paramKey='layout',
            label='Cilindri / disposizione',
            choices=Car.get_layout_choices(),
        )
        return layout

    def create_fuel_dict(self):
        fuel = dict(
            filterType='multiselect',
            paramKey='fuel',
            label='Alimentazione',
            choices=Car.get_fuel_choices(),
        )
        return fuel

    def create_traction_dict(self):
        traction = dict(
            filterType='multiselect',
            paramKey='traction',
            label='Trazione',
            choices=Car.get_traction_choices(),
        )
        return traction

    def create_agear_dict(self):
        agear = dict(
            filterType='multiselect',
            paramKey='agear',
            label='Tipo di cambio',
            choices=Car.get_agear_choices()
        )
        return agear

    def create_pollution_dict(self):
        pollution = dict(
            filterType='multiselect',
            paramKey='pollution',
            label='Classe emissioni',
            choices=Car.get_pollution_choices(),
        )
        return pollution

    def create_engine_loc_dict(self):
        engine_loc = dict(
            filterType='multiselect',
            paramKey='engine_loc',
            label='Posizione motore',
            choices=Car.get_engine_loc_choices(),
        )
        return engine_loc

    def create_doors_dict(self):
        doors = dict(
            filterType='multiselect',
            paramKey='doors',
            label='Porte',
            choices=Car.get_door_choices(),
        )
        return doors

    def create_year_dict(self):
        year_range = Car.get_year_range()
        year = dict(
            filterType='range',
            paramKey='year',
            label='Anno di lancio',
            initialRange=year_range,
            range=year_range,
            unityOfMeasure='anno',
        )
        return year

    def create_luggage_dict(self):
        luggage_range = Car.get_luggage_range()
        luggage = dict(
            filterType='range',
            paramKey='luggage',
            label='Bagagliaio',
            initialRange=luggage_range,
            range=luggage_range,
            unityOfMeasure='litri',
        )
        return luggage

    def create_tank_dict(self):
        tank_range = Car.get_tank_range()
        tank = dict(
            filterType='range',
            paramKey='tank',
            label='Serbatoio',
            initialRange=tank_range,
            range=tank_range,
            unityOfMeasure='litri',
        )
        return tank

    def create_weight_dict(self):
        weight_range = Car.get_weight_range()
        weight = dict(
            filterType='range',
            paramKey='weight',
            label='Massa',
            initialRange=weight_range,
            range=weight_range,
            unityOfMeasure='kg',
        )
        return weight

    def create_length_dict(self):
        length_range = Car.get_length_range()
        length = dict(
            filterType='range',
            paramKey='length',
            label='Lunghezza',
            initialRange=length_range,
            range=length_range,
            unityOfMeasure='cm',
        )
        return length

    def create_width_dict(self):
        width_range = Car.get_width_range()
        width = dict(
            filterType='range',
            paramKey='width',
            label='Larghezza',
            initialRange=width_range,
            range=width_range,
            unityOfMeasure='cm',
        )
        return width

    def create_height_dict(self):
        height_range = Car.get_height_range()
        height = dict(
            filterType='range',
            paramKey='height',
            label='Altezza',
            initialRange=height_range,
            range=height_range,
            unityOfMeasure='cm',
        )
        return height

    def create_displ_dict(self):
        displ_range = Car.get_displ_range()
        displ = dict(
            filterType='range',
            paramKey='displ',
            label='Cilindrata',
            initialRange=displ_range,
            range=displ_range,
            unityOfMeasure='cc',
        )
        return displ

    def create_power_cv_dict(self):
        power_cv_range = Car.get_power_cv_range()
        power_cv = dict(
            filterType='range',
            paramKey='power_cv',
            label='Potenza massima',
            initialRange=power_cv_range,
            range=power_cv_range,
            unityOfMeasure='CV',
        )
        return power_cv

    def create_total_power_cv_dict(self):
        total_power_cv_range = Car.get_total_power_cv_range()
        total_power_cv = dict(
            filterType='range',
            paramKey='total_power_cv',
            label='Potenza massima di sistema',
            initialRange=total_power_cv_range,
            range=total_power_cv_range,
            unityOfMeasure='CV',
        )
        return total_power_cv

    def create_torque_max_dict(self):
        torque_max_range = Car.get_torque_max_range()
        torque_max = dict(
            filterType='range',
            paramKey='torque_max',
            label='Coppia massima',
            initialRange=torque_max_range,
            range=torque_max_range,
            unityOfMeasure='Nm',
        )
        return torque_max

    def create_total_torque_Nm_dict(self):
        total_torque_Nm_range = Car.get_total_torque_Nm_range()
        total_torque_Nm = dict(
            filterType='range',
            paramKey='total_torque_Nm',
            label='Coppia massima di sistema',
            initialRange=total_torque_Nm_range,
            range=total_torque_Nm_range,
            unityOfMeasure='Nm',
        )
        return total_torque_Nm

    def create_speed_dict(self):
        speed_range = Car.get_speed_range()
        speed = dict(
            filterType='range',
            paramKey='speed',
            label='Velocità massima',
            initialRange=speed_range,
            range=speed_range,
            unityOfMeasure='km/h',
        )
        return speed

    def create_acc_dict(self):
        acc_range = Car.get_acc_range()
        acc = dict(
            filterType='range',
            paramKey='acc',
            label='Accelerazione (0-100 km/h)',
            initialRange=acc_range,
            range=acc_range,
            unityOfMeasure='secondi',
        )
        return acc

    def create_co2_dict(self):
        co2_range = Car.get_co2_range()
        co2 = dict(
            filterType='range',
            paramKey='co2',
            label='Emissioni CO2',
            initialRange=co2_range,
            range=co2_range,
            unityOfMeasure='g/km',
        )
        return co2

    def create_cons_mix_dict(self):
        cons_mix_range = Car.get_cons_mix_range()
        cons_mix = dict(
            filterType='range',
            paramKey='cons_mix',
            label='Consumo medio',
            initialRange=cons_mix_range,
            range=cons_mix_range,
            unityOfMeasure='l/100 km',
        )
        return cons_mix

    def create_average_electric_consumption_dict(self):
        average_electric_consumption_range = Car.get_average_electric_consumption_range()
        average_electric_consumption = dict(
            filterType='range',
            paramKey='average_electric_consumption',
            label='Consumo medio EV',
            initialRange=average_electric_consumption_range,
            range=average_electric_consumption_range,
            unityOfMeasure='kWh/100 km',
        )
        return average_electric_consumption

    def create_kml_mix_dict(self):
        kml_mix_range = Car.get_kml_mix_range()
        kml_mix = dict(
            filterType='range',
            paramKey='kml_mix',
            label='Percorrenza media',
            initialRange=kml_mix_range,
            range=kml_mix_range,
            unityOfMeasure='km/l',
        )
        return kml_mix

    def create_electric_engine_power_kw_dict(self):
        electric_engine_power_kw_range = Car.get_electric_engine_power_kw_range()
        kml_mix = dict(
            filterType='range',
            paramKey='electric_engine_power_kw',
            label='Batteria EV',
            initialRange=electric_engine_power_kw_range,
            range=electric_engine_power_kw_range,
            unityOfMeasure='kW',
        )
        return kml_mix

    def create_total_power_cv_dict(self):
        total_power_cv_range = Car.get_total_power_cv_range()
        total_power_cv = dict(
            filterType='range',
            paramKey='total_power_cv',
            label='Potenza massima di sistema',
            initialRange=total_power_cv_range,
            range=total_power_cv_range,
            unityOfMeasure='CV',
        )
        return total_power_cv

    def create_electric_autonomy_in_km_dict(self):
        electric_autonomy_in_km_range = Car.get_electric_autonomy_in_km_range()
        electric_autonomy_in_km = dict(
            filterType='range',
            paramKey='electric_autonomy_in_km',
            label='Autonomia EV',
            initialRange=electric_autonomy_in_km_range,
            range=electric_autonomy_in_km_range,
            unityOfMeasure='km',
        )
        return electric_autonomy_in_km

    def create_driving_dict(self):
        driving = dict(
            filterType='boolean',
            paramKey='driving',
            label='Neopatentati',
            value=None,
        )
        return driving

    def create_area_c_dict(self):
        area_c = dict(
            filterType='boolean',
            paramKey='area_c',
            label='Area C',
            value=None,
        )
        return area_c

    def handle(self, *scripts, **options):
        # verbose = int(options['verbosity'])
        data = dict(
            brand=self.create_brand_dict(),
            price=self.create_price_dict(),
            shape=self.create_shape_dict(),
            target=self.create_target_dict(),
            seat=self.create_seat_dict(),
            engine=self.create_engine_dict(),
            layout=self.create_layout_dict(),
            fuel=self.create_fuel_dict(),
            traction=self.create_traction_dict(),
            agear=self.create_agear_dict(),
            pollution=self.create_pollution_dict(),
            engine_loc=self.create_engine_loc_dict(),
            doors=self.create_doors_dict(),
            year=self.create_year_dict(),
            luggage=self.create_luggage_dict(),
            tank=self.create_tank_dict(),
            weight=self.create_weight_dict(),
            lenght=self.create_length_dict(),
            width=self.create_width_dict(),
            height=self.create_height_dict(),
            displ=self.create_displ_dict(),
            power_cv=self.create_power_cv_dict(),
            total_power_cv=self.create_total_power_cv_dict(),
            torque_max=self.create_torque_max_dict(),
            total_torque_Nm=self.create_total_torque_Nm_dict(),
            speed=self.create_speed_dict(),
            acc=self.create_acc_dict(),
            co2=self.create_co2_dict(),
            cons_mix=self.create_cons_mix_dict(),
            average_electric_consumption=self.create_average_electric_consumption_dict(),
            electric_autonomy_in_km=self.create_electric_autonomy_in_km_dict(),
            kml_mix=self.create_kml_mix_dict(),
            driving=self.create_driving_dict(),
            area_c=self.create_area_c_dict(),
            electric_engine_power_kw=self.create_electric_engine_power_kw_dict(),
        )
        if options.get('debug'):
            print "log DEBUG too"
            logger.setLevel(logging.DEBUG)
        json_data = json.dumps(data, indent=4, sort_keys=True)

        json_path = os.path.join(
            os.path.dirname(settings.PROJECT_PATH),
            ".src/publieditor.carsearch/publieditor/carsearch/static/carsearch/js/new/configServices.js",
        )

        js_text = """
/* global angular */

angular.module('configServices', [])

.factory('filtersData', function (){
    var data = %s;
    return data;
});""" % json_data
        out_file = open(json_path, "w")
        out_file.write(js_text)
