/* global angular */

angular.module('controllers', [])

.controller('FormCtrl', ['$scope', 'CarFilters', 'ResultService', 'InitService', '$location', '$anchorScroll',
    function ($scope, filtersManager, ResultService, InitService, $location, $anchorScroll) {
        'use strict';

        $scope.applicationLoading = true;
        $scope.resultsLoading = false;

        $scope.result = null;
        $scope.numberOfVersions = null;

        $scope.isResultsUpdateActive = true;
        $scope.suspendResultsUpdate = function () {
            $scope.isResultsUpdateActive = false;
        };
        $scope.resumeResultsUpdate = function () {
            $scope.isResultsUpdateActive = true;
            $scope.displayResults();
        };

        $scope.filtersManager = filtersManager;

        $scope.scrollToTop = function () {
            $location.hash('top');
            $anchorScroll();
        };

        InitService.getBrandsAndModels(
            function (response) {
                $scope.brands = response.brands;
                $scope.models = response.models;
                $scope.applicationLoading = false;

                $scope.displayNumberOfVersions = function () {
                    var params = $scope.filtersManager.getParams();
                    ResultService.countVersions(params, function (numberOfVersions) {
                        $scope.numberOfVersions = numberOfVersions;
                    });
                };

                // Todo: update automatically the results on filters change, but only on mouseup in sliders.
                $scope.$watch('filtersManager.getParams()', function (newParams, oldParams, scope) {
                    if ($scope.isResultsUpdateActive) {
                        scope.displayResults();
                    }
                    scope.activeParams = newParams;
                }, true);

                /* Result */

                $scope.displayResults = function () {
                    $scope.resultsLoading = true;
                    var params = $scope.filtersManager.getParams();
                    ResultService.getResult(params,
                        function (result) { // Success callback
                            var len = result.count; // It isn't the number of models but the number of versions! See the Api.
                            if (len < 1000 && len >= 100) {
                                $scope.showAllModels(result);
                            } else if (len < 100 ) {
                                $scope.showAllVersions(result);
                            }
                            $scope.resultsLoading = false;
                            $scope.result = result ;
                        }, function (result) { // Error callback
                            // TODO: handle the error somehow
                            $scope.resultsLoading = false;
                    });
                };

                $scope.toggleModels = function (brand) {
                    brand.modelsVisible = !brand.modelsVisible;
                };

                $scope.toggleVersions = function (model) {
                    if (model.versionsVisible === undefined) {
                        // If versions have not been loaded yet, load them
                        model.versionsLoading = true;
                        var promise = model.getVersions();
                        promise.then(function (result) {
                            model.versionsLoading = false;
                            model.versions = result.data;
                            model.versionsVisible = true;
                        })
                    } else {
                        model.versionsVisible = !model.versionsVisible;
                    }
                };

                $scope.showAllModels = function (result) {
                    angular.forEach(result.brands, function (brand) {
                        brand.modelsVisible = true;
                    });
                };

                $scope.showAllVersions = function (result) {
                    $scope.showAllModels(result);
                    angular.forEach(result.brands, function (brand) {
                        angular.forEach(brand.models, function (model) {
                            $scope.toggleVersions(model);
                        })
                    })
                };

                /* Url related stuff */

                // Utility to write to url the currently active filters (not used)
                $scope.loadFiltersToUrl = function () {
                    $location.search($scope.activeParams);
                };

                $scope.displaySharableUrl = function () {
                    // This is a ugly but very effective trick for building and getting the url "without" displaying it in the url bar.
                    // It would be otherwise much harder... See: http://stackoverflow.com/a/21675273
                    if ($scope.activeParams) {
                        $location.search($scope.activeParams);
                        // Encode &s because: http://www.htmlhelp.com/tools/validator/problems.html#amp
                        $scope.sharableUrl = encodeURIComponent($location.absUrl());
                        $location.search({});
                    } else {
                        $scope.sharableUrl = $location.absUrl();
                    }
                };

                $scope.setFiltersFromUrl = function () {
                    $scope.filtersManager.setFiltersFromUrl();
                };

                /* Share button */

                $scope.sharePaneVisible = false;
                $scope.toggleSharePane = function () {
                    $scope.sharePaneVisible = !$scope.sharePaneVisible;
                };

                function init () {
                    $scope.setFiltersFromUrl();
                    $scope.displayResults();
                }
                init();

            }, function (response) {
                // TODO: handle the error somehow
                $scope.applicationLoading = false;
            }
        );

    }
]);