angular.module('directives', [])


/* ~~~ Filters pane directories ~~~ */

// Level 1. Main container.
.directive('filterTabsContainer', function ($document) {
    return {
        restrict: 'A',
        scope: true,
        templateUrl: '/prezzi-auto/static/carsearch/partials/filter-tabs-container.html',
        // Create an API for downstream directives. See: http://stackoverflow.com/questions/16866749/access-parent-scope-in-transcluded-directive
        controller: function ($scope) {
            // Todo: there must be a better way instead of this bridge
            this.filters = $scope.filtersManager.filters;
            this.displayResults = $scope.displayResults;
            this.suspendResultsUpdate = $scope.suspendResultsUpdate;
            this.resumeResultsUpdate = $scope.resumeResultsUpdate;
        },
        link: function ($scope, element, attrs) {

            // Filters tabs container visibility manager
            $('.filter-tabs-container-toggle-button').bind('click', function (event) {
                $scope.$apply(function () {
                    $scope.toggle();
                });
            });

            // Tab expansion manager. Coordinates the tabs being expanded or not.
            $scope.visible = true;
            $scope.toggle = function () {
                $scope.visible = !$scope.visible;
            };
            $scope.close = function () {
                $scope.visible = false;
            }

        }
    };
})

// Level 2. Tab
.directive('filterTab', function () {
    return {
        restrict: 'A',
        templateUrl: '/prezzi-auto/static/carsearch/partials/filter-tab.html',
        transclude: true,
        scope: {},
        link: function (scope, element, attrs) {

            scope.title = attrs.filterTab;

            scope.expanded = scope.$eval(attrs.expanded);
            scope.toggle = function () {
                scope.expanded = !scope.expanded;
            };

        }
    };
})

// Level 3. Widget. Every control has a single model (price, weight, speed...) and contains multiple controls
.directive('filterWidget', function () {
    return {
        restrict: 'A',
        scope: {
            hideLabel: '@'
        },
        require: '^filterTabsContainer',
        transclude: true,
        link: function (scope, element, attrs, controller, transclude) {

            var filterId = attrs.filterWidget;
            scope.filter = controller.filters[filterId];

            scope.slider = {
                options: {
                    range: true,
                    start: function (event, ui) {
                        controller.suspendResultsUpdate();
                    },
                    stop: function (event, ui) {
                        controller.resumeResultsUpdate();
                    }
                }
            };

            // This just crops the content of the element, compiles against the new scope (not the parent).
            // See: http://angular-tips.com/blog/2014/03/transclusion-and-scopes/
            transclude(scope, function(clone, scope) {
                element.append(clone);
            });

        }
    };
})


// Level 4. Controls

// Loosely inspired by https://github.com/isteven/angular-multi-select
//Other similar project https://github.com/amitava82/angular-multiselect
.directive('multiselectFilterControl', function () {
    return {
        restrict: 'A',
        scope: true,
        templateUrl: '/prezzi-auto/static/carsearch/partials/multiselect-filter-control.html',
        transclude: true,
        link: function (scope, element, attrs, controller, transclude) {

            scope.colWidth = (24 / attrs.columns) || 12;

            scope.toggleChoice = function (choice, e) {
                choice.active = !choice.active;
            };

            // This just crops the content of the element, compiles against the new scope (not the parent).
            // See: http://angular-tips.com/blog/2014/03/transclusion-and-scopes/
            transclude(scope, function(clone, scope) {
                //element.empty();
                //element.append(clone);
            });

        }
    };
})

.directive('rangeFilterControl', function () {
    return {
        restrict: 'A',
        link: function (scope, element, attrs, controller, transclude) {

            scope.toggleRange = function (range, e) {
                if (scope.filter.range[0] === range[0] && scope.filter.range[1] === range[1]) {
                    scope.filter.range = scope.filter.initialRange;
                } else {
                    scope.filter.range = range;
                }
            };

        }
    };
})

.directive('booleanFilterControl', function () {
    return {
        restrict: 'A',
        link: function (scope, element, attrs, controller, transclude) {

            scope.toggleBoolean = function (value, e) {
                if (scope.filter.value === 0 || scope.filter.value === null) {
                    scope.filter.value = 1;
                } else if (scope.filter.value === 1) {
                    scope.filter.value = 0;
                }
            };
        }
    };
})


/*
.directive('rangeFilterControl')...
 */
.directive('suspendResults', function () {
    return {
        restrict: 'A',
        require: '^filterTabsContainer',
        link: function (scope, element, attrs, controller, transclude) {
            element.bind('mousedown', function () {
                controller.suspendResultsUpdate();
            });
            element.bind('mouseup', function () {
                controller.resumeResultsUpdate();
            });
        }
    };

})

/* ~~~ Results directories ~~~ */

.directive('resultGrid', function ($http) {
    "use strict";
    return {
        restric: 'A',
        templateUrl: function (elem, attrs) {
            return '/prezzi-auto/static/carsearch/partials/result.html';
        },
        link: function (scope, element, attrs) {

            function addBadge(resp, $elem) {
                var $parent = $elem.parent(),
                    $badge = $parent.find('.badge');

                if ($badge.length === 0) {
                    $parent.append('<span class="badge badge-success"></span>')
                }
                $badge.text(resp.length);
            }

            scope.updateFavourites = function () {
                $http.get('/prezzi-auto/get-favourites')
                    .success(function (response) {
                        scope.favourites = response;
                        addBadge(response, $('#preferite'));

                })
            };
            scope.updateFavourites();

            scope.updateCompares = function () {
                $http.get('/prezzi-auto/get-compare')
                    .success(function (response) {
                        scope.compares = response;
                        addBadge(response, $('#confronta'));
                })
            };
            scope.updateCompares();

        }
    };
})

.directive('favouriteIcon', function ($http) {
    "use strict";
    return {
        restric: 'A',
        link: function (scope, element, attrs) {

            scope.toggleFavourite = function (versionId) {
                if (element.hasClass('is_favourite')) {
                    $http.get('/prezzi-auto/remove-from-favourites?car_id=' + versionId)
                      .success(function() {
                        scope.updateFavourites();
                    });
                } else {
                    $http.get('/prezzi-auto/add-to-favourites?car_id=' + versionId)
                      .success(function () {
                        scope.updateFavourites();
                    })
                }
            }

        }
    };
})

.directive('compareIcon', function ($http) {
    "use strict";
    return {
        restric: 'A',
        link: function (scope, element, attrs) {

            scope.toggleCompare = function (versionId) {
                if (element.hasClass('is_to_compare')) {
                    $http.get('/prezzi-auto/remove-from-compare?car_id=' + versionId)
                      .success(function() {
                        scope.updateCompares();
                    });
                } else {
                    $http.get('/prezzi-auto/add-to-compare?car_id=' + versionId)
                      .success(function () {
                        scope.updateCompares();
                    })
                }
            }

        }
    };
})


/* ~~~ Helper directories ~~~ */

// from https://github.com/mattosborn/angular-sticky
.directive("sticky", function ($window) {
    return {
        link: function (scope, element, attrs) {

            var $win = angular.element($window);

            if (scope._stickyElements === undefined) {
                scope._stickyElements = [];

                $win.bind("scroll.sticky", function (e) { // Namespaced event name. See: http://api.jquery.com/on/#event-names
                    var pos = $win.scrollTop();
                    for (var i = 0; i < scope._stickyElements.length; i++) {

                        var item = scope._stickyElements[i];

                        if (!item.isStuck && pos > item.start) {
                            item.element.addClass("stuck");
                            item.isStuck = true;

                            if (item.placeholder) {
                                item.placeholder = angular.element("<div></div>")
                                    .css({height: item.element.outerHeight() + "px"})
                                    .insertBefore(item.element);
                            }
                        }
                        else if (item.isStuck && pos < item.start) {
                            item.element.removeClass("stuck");
                            item.isStuck = false;

                            if (item.placeholder) {
                                item.placeholder.remove();
                                item.placeholder = true;
                            }
                        }
                    }
                });

                var recheckPositions = function () {
                    for (var i = 0; i < scope._stickyElements.length; i++) {
                        var item = scope._stickyElements[i];
                        if (!item.isStuck) {
                            item.start = item.element.offset().top;
                        } else if (item.placeholder) {
                            item.start = item.placeholder.offset().top;
                        }
                    }
                };
                $win.bind("load", recheckPositions);
                $win.bind("resize", recheckPositions);
            }

            var item = {
                element: element,
                isStuck: false,
                placeholder: attrs.usePlaceholder !== undefined,
                start: element.offset().top
            };

            scope._stickyElements.push(item);

        }
    };
})

/*
.directive('sharableUrl', function ($document) {
    "use strict";
    return {
        restric: 'A',
        templateUrl: '/static/carsearch/partials/sharableUrl.html',
        link: function (scope, element, attrs) {

            element.find('.button').bind('click', function (event) {
                scope.displaySharableUrl();
                event.stopPropagation();
                scope.$apply(function () {
                    scope.sharableUrlIsVisible = true;
                });
            });

            element.find('.sharable-url-container').bind('click', function (event) {
                event.stopPropagation();
            });

            $document.bind('click', function () {
                scope.$apply(function () {
                    scope.sharableUrlIsVisible = false;
                });
            });


        }
    };
})
*/

.directive('updateFavouritesAndCompare', function ($document) {
    "use strict";
    return {
        restric: 'A',
        link: function (scope, element, attrs) {
            //ContentApp.updateCompareFromServer();


        }
    };
});
