/* global angular */

angular.module('services', [])

.factory('BrandsService', function ($http, $q) {

    var brands = null;

    var Brand = function (brand) {
        angular.extend(this, brand);
    };

    var Brands = function (response) {
        var i;
        this.data = [];
        for (i = 0; i < response.data.length; i++) {
            this.data.push(new Brand(response.data[i]));
        }
    };

    Brands.prototype.getBrandById = function (id) {
        var i = 0,
            len = this.data.length,
            data = this.data;
        for (; i < len; i++) {
            if (+data[i].id === +id) {
                return data[i];
            }
        }
        return null; // In case I don't find the requested id.
    };


    var promise = $http.get('/prezzi-auto/car-search/api/brands');

    return {
        promise: promise,
        Brands: Brands
    };

})

.factory('ModelsService', function ($http, $q) {

    var models = null;

    var Model = function (model) {
        angular.extend(this, model);
    };

    Model.prototype.getNumberOfVersionsForResult = function () {
        var number = 0;
        angular.forEach(this.versions, function (version) {
            number++;
        });
        return number;
    };

    // Todo: shouldn't this be a Results' method???
    Model.prototype.getMinPriceForResult = function () {
        var minPrice;
        angular.forEach(this.versions, function (version) {
            var price = version.price;
            if (!minPrice || (minPrice > price)) {
                minPrice = price;
            }
        });
        return minPrice;
    };

    // Todo: shouldn't this be a Results' method???
    Model.prototype.getMaxPriceForResult = function () {
        var maxPrice;
        angular.forEach(this.versions, function (version) {
            var price = version.price;
            if (!maxPrice || (maxPrice < price)) {
                maxPrice = price;
            }
        });
        return maxPrice;
    };

    var Models = function (response) {
        var i;
        this.data = [];
        for (i = 0; i < response.data.length; i++) {
            this.data.push(new Model(response.data[i]));
        }
    };

    Models.prototype.getModelById = function (id) {
        var i = 0,
            len = this.data.length,
            data = this.data;
        for (; i < len; i++) {
            if (+data[i].id === +id) {
                return data[i];
            }
        }
        return null; // In case I don't find the requested id.
    };

    var promise = $http.get('/prezzi-auto/car-search/api/models');

    return {
        promise: promise,
        Models: Models
    };

})

        // INIT
.factory('InitService', ['$q', 'BrandsService', 'ModelsService', function ($q, BrandsService, ModelsService) {
    var InitObj = {
        a: 'ciao',
        b: 'ciao belli'
    };
    InitObj.getBrandsAndModels = function (successcb, errorcb) {
        $q.all([BrandsService.promise, ModelsService.promise]).then(
            function (responses) {
                var results = {
                    brands: new BrandsService.Brands(responses[0]),
                    models: new ModelsService.Models(responses[1])
                };
                successcb(results);
            }, function (responses) {
                errorcb(responses);
            }
        );
    };

    return InitObj;

}])

.factory('CarFilters', ['$location', 'filtersData', function ($location, filtersData) {


    function Filter () {}

    Filter.prototype.isActive = function () {
        if (!this.isEmptyState()) {
            return true;
        }
    };

    Filter.prototype.deactivate = function () {
        this.setToEmptyState();
    }

    // Multiselect filters constructors

    function MultiselectFilter (options) {
        this.filterType = options.filterType;
        this.paramKey = options.paramKey;
        this.label = options.label;
        this.choices = options.choices;
    }

    MultiselectFilter.prototype = new Filter();

    MultiselectFilter.prototype.getActiveChoices = function () {

        var activeChoices = [];

        angular.forEach(this.choices, function (choice) {
            if (choice.active) {
                activeChoices.push(choice);
            }
        });
        // Todo: very ugly...
        if (activeChoices.length > 0) {
            return activeChoices;
        }
    };

    MultiselectFilter.prototype.isEmptyState = function () {
        if (!this.getActiveChoices()) {
            return true;
        }
    };

    MultiselectFilter.prototype.setToEmptyState = function () {
        angular.forEach(this.choices, function (choice) {
            choice.active = false;
        })
    };

    MultiselectFilter.prototype.getParams = function () {

        var param = {},
            paramValues = [],
            activeChoices = this.getActiveChoices();

        if (this.isActive()) {
            angular.forEach(activeChoices, function (choice) {
                paramValues.push(choice.value);
            });
            param[this.paramKey] = paramValues;
            return param;
        }
    };

    MultiselectFilter.prototype.setFromUrl = function (urlParams) {
        var that = this,
            values;
        angular.forEach(urlParams, function (value, key) {
            // If part of the param's key coincides with the filter's paramsKey name...
            if (key.indexOf(that.paramKey) > -1) {
                values = angular.isArray(value) ? value : [value]; // Force the param value to be an array (which already is if it's multiple. Like shape=compact&shape=pickup)
                // Value of the params object should be an array
                angular.forEach(that.choices, function (choice) {
                    angular.forEach(values, function (value) {
                        if (choice.value === value) {
                            choice.active = true;
                        }
                    });
                });
            }
        });
    };


    // Range filters constructor

    function RangeFilter (options) {
        this.filterType = options.filterType;
        this.paramKey = options.paramKey;
        this.label = options.label;
        this.initialRange = options.initialRange;
        this.range = options.range;
        this.unityOfMeasure = options.unityOfMeasure;
    }

    RangeFilter.prototype = new Filter();

    RangeFilter.prototype.setToEmptyState = function () {
        this.range = angular.copy(this.initialRange);
    }

    RangeFilter.prototype.getParams = function () {
        if (this.isActive()) {
            var params = {};
            params['min_' + this.paramKey] = this.range[0];
            params['max_' + this.paramKey] = this.range[1];
            return params;
        }
    };

    RangeFilter.prototype.setFromUrl = function (urlParams) {
        var that = this;
        angular.forEach(urlParams, function (value, key) {
            // If part of the param's key coincides with the filter's paramsKey name...
            if (key.indexOf(that.paramKey) > -1) {
                if (key.substring(0, 4) === 'min_') {
                    that.range[0] = value;
                } else if (key.substring(0, 4) === 'max_') {
                    that.range[1] = value;
                }
            }
        });
    };

    RangeFilter.prototype.isEmptyState = function () {
        if (this.range[0] === this.initialRange[0] &&
            this.range[1] === this.initialRange[1]) {
            return true;
        }
    };


    // Boolean filters constructor

    function BooleanFilter (options) {
        this.filterType = options.filterType;
        this.paramKey = options.paramKey;
        this.label = options.label;
        this.value = null;
    }

    BooleanFilter.prototype = new Filter();

    BooleanFilter.prototype.setToEmptyState = function () {
        this.value = null; // Todo: ???
    }

    BooleanFilter.prototype.getParams = function () {
        if (this.isActive()) {
            var param = {};
            param[this.paramKey] = this.value;
            return param;
        }
    };

    BooleanFilter.prototype.setFromUrl = function (urlParams) {
        var that = this;
        angular.forEach(urlParams, function (value, key) {
            if (key === that.paramKey) {
                that.value = value;
            }
        });
    };

    BooleanFilter.prototype.isEmptyState = function () {
        if (this.value === null) {
            return true;
        }
    };


    // Define a skeleton filter factory
    function FilterFactory () {}

    // Our factory method fo creating new Vehicle instances
    FilterFactory.prototype.createFilter = function (options) {

        if (options.filterType === "range") {
            this.filterClass = RangeFilter;
        } else if (options.filterType === "multiselect") {
            this.filterClass = MultiselectFilter;
        } else if (options.filterType === "boolean") {
            this.filterClass = BooleanFilter;
        }

        return new this.filterClass(options);
    };

    var factory = new FilterFactory();

    function FiltersManager (initialParamsObj) {

        // The 'that' variable is a workaround. See: Douglas Crockford's JS The Good Parts, 4.3.2. The Function Invocation Pattern
        var that = this;
        this.length = 0;
        this.filters = {};

        if (initialParamsObj !== undefined) {
            angular.forEach(initialParamsObj, function (param) {
                that.params[param] = initialParamsObj[param];
                that.length++;
            });
        }

        angular.forEach(filtersData, function (filterOptions) {
            var filter = factory.createFilter(filterOptions);
            that.filters[filter.paramKey] = filter;
        });

        // No need to attach methods to the prototype since it's going to be a singleton.

        this.getParams = function () {
            var params = {};
            angular.forEach(this.filters, function (filter) {
                // Every filter type must implement its own getParams method.
                var filterParams = filter.getParams();
                if (filterParams) {
                    angular.extend(params, filterParams);
                }
            });
            // Todo: is this correct?
            if (angular.equals({}, params)) {
                return null;
            } else {
                return params;
            }
        };

        this.setFiltersFromUrl = function () {
            var searchParams = $location.search();
            angular.forEach(this.filters, function (filter) {
                filter.setFromUrl(searchParams);
            });
        };
    }

    return new FiltersManager();

}])


.factory('ResultService', ['$http', '$filter', 'InitService', 'CarFilters', function ($http, $filter, InitService, filtersManager) {

    var allBrands, allModels;

    InitService.getBrandsAndModels(
        function (response) {
            allBrands = response.brands;
            allModels = response.models;
        }, function (response) {
            // TODO: handle the error somehow
        }
    );

    function ResultModel (modelId, result) {
        angular.extend(this, allModels.getModelById(modelId));
        angular.extend(this, result);
        this.versions = [];
    };

    ResultModel.prototype.getVersions = function () {
        // Todo: this must be addressed more upstream
        var modelParams = {model: this.id},
            otherParams = (filtersManager.getParams() || {});
        var params = angular.extend(otherParams, modelParams);
        var promise = $http.get('/prezzi-auto/car-search/api/cars', { params: params })
        return promise;
    };

    function ResultBrand (brandId) {
        angular.extend(this, allBrands.getBrandById(brandId));
        this.models = [];
        this.modelsVisible = false;
    }

    function Result (responseData) {
        // This add additional other information (beside 'data') from the API response like 'count' and 'layout'.
        angular.extend(this, responseData);


        var brands = [],
            models = [],
            previousBrandId,
            previousBrand;

        angular.forEach(responseData.data, function (result) {
            var model = new ResultModel(result.model, result);
            if (model) {
                models.push(model);
            } else {
                console.log('Invalid model with id: ' + result.id);
            }
        });

        // Data must be currently ordered by brand, model.
        models = $filter('orderBy')(models, ['brand']);

        angular.forEach(models, function (model) {
            var brandId = model.brand,
                brand;
            if (brandId !== previousBrandId) {
                brand = new ResultBrand(brandId);
                if (brand) {
                    brand.models.push(model);
                    brands.push(brand);
                    previousBrandId = brandId;
                    previousBrand = brand;
                } else {
                    console.log('Model' + model.name + ' has no valid brand with id:' + brandId);
                }
            } else {
                previousBrand.models.push(model);
            }
        });

        this.brands = brands;
    }

    Result.prototype.getNumberOfVersionsForBrand = function (brand) {
        var number = 0;
        angular.forEach(brand.models, function (model) {
            number += model.numberOfVersionsForThisResult;
        });
        return number;
    };

    Result.prototype.getMinPriceForBrand = function (brand) {
        var price, minPrice;
        angular.forEach(brand.models, function (model) {
            price = model.minPrice;
            if (!minPrice || (minPrice > price)) {
                minPrice = price;
            }
        });
        return minPrice;
    };

    Result.prototype.getMaxPriceForBrand = function (brand) {
        var price, maxPrice;
        angular.forEach(brand.models, function (model) {
            price = model.maxPrice;
            if (!maxPrice || (maxPrice < price)) {
                maxPrice = price;
            }
        });
        return maxPrice;
    };

    // Inspired by: https://github.com/angular-app/angular-app/blob/master/client/vendor/mongolab/mongolab-resource.js
    var thenFactoryMethod = function (httpPromise, successcb, errorcb, isArray) {
        var scb = successcb || angular.noop;
        var ecb = errorcb || angular.noop;

        return httpPromise.then(function (response) {
            var result = new Result(response.data);
            scb(result, response.status, response.headers, response.config);
            return result;
        }, function (response) {
            ecb(undefined, response.status, response.headers, response.config);
            return undefined;
        });
    };

    var ResultService = {};

    ResultService.getResult = function (queryParams, successcb, errorcb) {

        $http.get(
            '/prezzi-auto/car-search/api/results',
            { params: queryParams }
        ).then(function (response) {
            var result = new Result(response.data);
            successcb(result, response.status, response.headers, response.config);
            return result;
        }, function (response) {
            errorcb(undefined, response.status, response.headers, response.config);
            return undefined;
        });

    };

    ResultService.countVersions = function (queryParams, successcb, errorcb) {

        $http.get(
            '/prezzi-auto/car-search/api/car-count',
            { params: queryParams }
        ).then(function (response) {
            successcb(response.data);
        });
    };

    return ResultService;
}]);