# -*- coding: utf-8 -*-

from django.conf import settings
from django.utils.translation import ugettext as _
from django.views.generic import TemplateView

from publieditor.content.models import CarModel, Hit


class CarSearchAppView(TemplateView):
    template_name = 'carsearch/car_search_app.html'

    @staticmethod
    def get_models_in_home():
        return CarModel.objects.filter(
            status__in=('online', 'runout'),
            new_entry__in=('side', 'home')
        ).order_by('-id')[:3]

    def get(self, request, *args, **kwargs):
        breadcrumbs = settings.PUBLIEDITOR_CONTENT_BREADCRUMBS + [
            {'title': _('ricerca mirata'), 'url': None}
        ]
        context = self.get_context_data(**kwargs)
        context['breadcrumbs'] = breadcrumbs
        context['debug'] = settings.DEBUG
        context['models_in_home'] = self.get_models_in_home()
        context['hits'] = Hit.objects.filter(status=1).order_by('ordering')[:3]
        return self.render_to_response(context)


class CarSearchAppViewNew(TemplateView):
    template_name = 'carsearch/car_search_app_new.html'